==> cptac-0.9.0-py3-none-any.whl: cptac/pancan/umichov.py <==
#   Copyright 2018 Samuel Payne sam_payne@byu.edu
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#       http://www.apache.org/licenses/LICENSE-2.0
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import pandas as pd
import numpy as np
import os
import warnings
import datetime

from cptac.dataset import Dataset
from cptac.dataframe_tools import *
from cptac.exceptions import FailedReindexWarning, PublicationEmbargoWarning, ReindexMapError


class UmichOv(Dataset):

    def __init__(self, no_internet, version):
        """Load all of the umichov dataframes as values in the self._data dict variable, with names as keys, and format them properly.

        Parameters:
        version (str, optional): The version number to load, or the string "latest" to just load the latest building. Default is "latest".
        no_internet (bool, optional): Whether to skip the index update step because it requires an internet connection. This will be skipped automatically if there is no internet at all, but you may want to manually skip it if you have a spotty internet connection. Default is False.
        """

        # Set some needed variables, and pass them to the parent Dataset class __init__ function

        # This keeps a record of all versions that the code is equipped to handle. That way, if there's a new data release but they didn't update their package, it won't try to parse the new data version it isn't equipped to handle.
        valid_versions = ["1.0"]

        data_files = {
            "1.0": ["Report_abundance_groupby=protein_protNorm=MD_gu=2.tsv",
                    "OV_sample_TMT_annotation_UMich_GENCODE34_0315.csv",
                    "Report_abundance_groupby=multi-site_protNorm=MD_gu=2.tsv"
                #"S039_BCprospective_observed_0920.tsv.gz",
                #"S039_BCprospective_imputed_0920.tsv.gz"
            ]
        }

        # Call the parent class __init__ function
        super().__init__(cancer_type="umichov", version=version, valid_versions=valid_versions, data_files=data_files, no_internet=no_internet)

        # Load the data into dataframes in the self._data dict
        loading_msg = f"Loading {self.get_cancer_type()} v{self.version()}"
        for file_path in self._data_files_paths: # Loops through files variable

            # Print a loading message. We add a dot every time, so the user knows it's not frozen.
            loading_msg = loading_msg + "."
            print(loading_msg, end='\r')

            path_elements = file_path.split(os.sep) # Get a list of the levels of the path
            file_name = path_elements[-1] # The last element will be the name of the file. We'll use this to identify files for parsing in the if/elif statements below
            
            
            if file_name == "Report_abundance_groupby=protein_protNorm=MD_gu=2.tsv":
                df = pd.read_csv(file_path, sep = "\t") 
                df = df.drop(columns = ['MaxPepProb', 'NumberPSM']) 
                df.Index = df.Index.apply(lambda x: x.split('|')[5]) # Get gene name from position in list of gene identifiers
                df = df.rename(columns = {'Index':'Name', 'Gene':'Database_ID'})
                df = df.set_index(['Name', 'Database_ID']) # set multiindex
                df = df.transpose()
                ref_intensities = df.loc["ReferenceIntensity"] # Get reference intensities to use to calculate ratios 
                df = df.subtract(ref_intensities, axis="columns") # Subtract reference intensities from all the values
                df = df.iloc[1:,:] # drop ReferenceIntensity row 
                df.index.name = 'Patient_ID'
    
                # Drop quality control and ref intensity cols
                drop_cols = ['JHU-QC', 'JHU-QC.1', 'JHU-QC.2', 'JHU-QC.3', 'JHU-QC.4',
                           'RefInt_PNNL-JHU-Ref-1', 'RefInt_PNNL-JHU-Ref-2',
                           'RefInt_PNNL-JHU-Ref-3', 'RefInt_PNNL-JHU-Ref-4',
                           'RefInt_PNNL-JHU-Ref-5', 'RefInt_PNNL-JHU-Ref-6',
                           'RefInt_PNNL-JHU-Ref-7', 'RefInt_PNNL-JHU-Ref-8',
                           'RefInt_PNNL-JHU-Ref-9', 'RefInt_PNNL-JHU-Ref-10',
                           'RefInt_PNNL-JHU-Ref-11', 'RefInt_PNNL-JHU-Ref-12']
                df = df.drop(drop_cols, axis = 'index')
                self._data["proteomics"] = df
                
            elif file_name == "OV_sample_TMT_annotation_UMich_GENCODE34_0315.csv":
                ov_map = pd.read_csv(file_path, sep = ",", index_col = 0)
                ov_map = ov_map[['sample']].reset_index()
                ov_map = ov_map.replace("#",".", regex = True)
                self._data["map_ids"] = ov_map
                
            elif file_name == "Report_abundance_groupby=multi-site_protNorm=MD_gu=2.tsv":
                df = pd.read_csv(file_path, sep = "\t") 
                df[['Protein_ID','Transcript_ID',"Database_ID","Havana_gene","Havana_transcript","Transcript","Name","Site"]] = df.Index.str.split("\\|",expand=True)
                df[['num1','num2',"num3","num4","num5","Site"]] = df.Site.str.split("_",expand=True) 
                df = df[df['Site'].notna()] # only keep columns with phospho site 
                df = df.set_index(["Name","Database_ID","Peptide","Site"]) 
                #drop columns not needed in df 
                df.drop([ 'Gene', "Index","num1","num2","num3","num4","num5","Havana_gene","Havana_transcript","MaxPepProb","Protein_ID","Transcript_ID","Transcript"], axis=1, inplace=True)
                self._data["phosphoproteomics"] = df

            '''
            elif file_name == "S039_BCprospective_observed_0920.tsv.gz":
                df = pd.read_csv(file_path, sep="\t")
                df = df.transpose()
                df.index.name = 'Patient_ID'
                df.columns.name = 'Name'
                df = average_replicates(df)
                df = df.sort_values(by=["Patient_ID"])
                self._data["proteomics"] = df  
                
            elif file_name == "S039_BCprospective_imputed_0920.tsv.gz":
                df = pd.read_csv(file_path, sep="\t")
                df = df.transpose()
                df.index.name = 'Patient_ID'
                df.columns.name = 'Name'
                df = average_replicates(df)
                df = df.sort_values(by=["Patient_ID"])
                self._data["proteomics_imputed"] = df'''
            
        # This code may need to be deleted 
        # The '.1' added to the df is added by pd.read_csv() when there are duplicate columns
        # The '.1' after the read on Box is read in '#1' in the mapping file do not represent the same sample
        # Emailed about duplicate columns and waiting for response
        '''    
        # Proteomics
        # Get Patient_IDs
        # slice mapping_df to include cancer specific aliquot_IDs 
        prot = self._data["proteomics"]
        index_list = list(prot.index)
        mapping_df = self._data["map_ids"]
        cancer_df = mapping_df.loc[mapping_df['index'].isin(index_list)]
        # Create dictionary with aliquot_ID as keys and patient_ID as values
        matched_ids = {}
        for i, row in cancer_df.iterrows():
            matched_ids[row['index']] = row['sample']
        prot = prot.reset_index()
        prot = prot.replace(matched_ids) # replace aliquot_IDs with Patient_IDs
        prot = prot.set_index('Patient_ID')
          
        # average duplicates - CHECK really duplicates (some blanks in file are normal, 
        # but no blanks in file for these ids)
        replicate_IDs = ['01OV029', '15OV001', '17OV002']
        for rep in replicate_IDs:
            id_df = prot[prot.index.str.contains(rep)] 
            vals = list(id_df.mean(axis=0)) # average replicates and store in list 
            prot = prot.drop(index = rep) # drop both replicates so can add new row with averages
            prot.loc[rep] = vals'''
        
        prot = self._data["proteomics"]
        prot.index = prot.index.str.replace('-T$','', regex = True)
        prot.index = prot.index.str.replace('-N$','.N', regex = True)
        
        # Sort
        normal = prot.loc[prot.index.str.contains('\.N$', regex = True)]
        normal = normal.sort_values(by=["Patient_ID"])
        tumor = prot.loc[~ prot.index.str.contains('\.N$', regex = True)]
        tumor = tumor.sort_values(by=["Patient_ID"])
        all_prot = tumor.append(normal)  
        self._data["proteomics"] = all_prot
                
        
        ## phosphoproteomics 
        phos = self._data["phosphoproteomics"]
        ovmap = self._data["map_ids"]
        ovmap = ovmap.set_index("index")
        ovmap_dict = ovmap.to_dict()["sample"]
        
        phos = phos.rename(columns = ovmap_dict)# rename NAT ID columns with .N 
        phos = phos.T #transpose df 
        ref_intensities = phos.loc["ReferenceIntensity"]# Get reference intensities to use to calculate ratios 
        phos = phos.subtract(ref_intensities, axis="columns") # Subtract reference intensities from all the values, to get ratios
        phos = phos.iloc[1:,:] # drop ReferenceIntensity row 
        
        phos.index = phos.index.str.replace('-T$','', regex = True)
        phos.index = phos.index.str.replace('-N$','.N', regex = True)
        
        drop_cols_phos = ['JHU-QC','JHU-QC-1-Q','JHU-QC-2-Q','JHU-QC-3-Q','JHU-QC-4-Q','RefInt_PNNL-JHU-Ref-1-R',
 'RefInt_PNNL-JHU-Ref-2-R','RefInt_PNNL-JHU-Ref-3-R','RefInt_PNNL-JHU-Ref-4-R','RefInt_PNNL-JHU-Ref-5-R',
 'RefInt_PNNL-JHU-Ref-6-R','RefInt_PNNL-JHU-Ref-7-R','RefInt_PNNL-JHU-Ref-8-R','RefInt_PNNL-JHU-Ref-9-R',
 'RefInt_PNNL-JHU-Ref-10-R','RefInt_PNNL-JHU-Ref-11-R','RefInt_PNNL-JHU-Ref-12-R']

          # Drop quality control and ref intensity cols
        phos = phos.drop(drop_cols_phos, axis = 'index')
     
        # average duplicates - CHECK really duplicates (some blanks in file are normal, 
        # but no blanks in file for these ids)
        replicate_IDs = ['01OV029', '15OV001', '17OV002']
        for rep in replicate_IDs:
            id_df = phos[phos.index.str.contains(rep)] 
            vals = list(id_df.mean(axis=0)) # average replicates and store in list 
            phos = phos.drop(index = rep) # drop both replicates so can add new row with averages
            phos.loc[rep] = vals
        
        self._data["phosphoproteomics"] = phos
        
        
        
        print(' ' * len(loading_msg), end='\r') # Erase the loading message
        formatting_msg = "Formatting dataframes..."
        print(formatting_msg, end='\r')

        # Get a union of all dataframes' indices, with duplicates removed
        ###FILL: If there are any tables whose index values you don't want
        ### included in the master index, pass them to the optional 'exclude'
        ### parameter of the unionize_indices function. This was useful, for
        ### example, when some datasets' followup data files included samples
        ### from cohorts that weren't in any data tables besides the followup
        ### table, so we excluded the followup table from the master index since
        ### there wasn't any point in creating empty representative rows for
        ### those samples just because they existed in the followup table.
#        master_index = unionize_indices(self._data) 

        # Use the master index to reindex the clinical dataframe, so the clinical dataframe has a record of every sample in the dataset. Rows that didn't exist before (such as the rows for normal samples) are filled with NaN.
#        new_clinical = self._data["clinical"]
#        new_clinical = new_clinical.reindex(master_index)

        # Add a column called Sample_Tumor_Normal to the clinical dataframe indicating whether each sample was a tumor or normal sample. Use a function from dataframe_tools to generate it.

        ###FILL: Your dataset should have some way that it marks the Patient IDs
        ### of normal samples. The example code below is for a dataset that
        ### marks them by putting an 'N' at the beginning of each one. You will
        ### need to write a lambda function that takes a given Patient_ID string
        ### and returns a bool indicating whether it corresponds to a normal
        ### sample. Pass that lambda function to the 'normal_test' parameter of
        ### the  generate_sample_status_col function when you call it. See 
        ### cptac/dataframe_tools.py for further function documentation.
        ###START EXAMPLE CODE###################################################
#        sample_status_col = generate_sample_status_col(new_clinical, normal_test=lambda sample: sample[0] == 'N')
        ###END EXAMPLE CODE#####################################################

#        new_clinical.insert(0, "Sample_Tumor_Normal", sample_status_col)

        # Replace the clinical dataframe in the data dictionary with our new and improved version!
#        self._data['clinical'] = new_clinical

        # Edit the format of the Patient_IDs to have normal samples marked the same way as in other datasets. 
        
        ###FILL: You will need to pass the proper parameters to correctly
        ### reformat the patient IDs in your dataset. The standard format is to
        ### have the string '.N' appended to the end of the normal patient IDs,
        ### e.g. the  normal patient ID corresponding to C3L-00378 would be
        ### C3L-00378.N (this way we can easily match two samples from the same
        ### patient). The example code below is for a dataset where all the
        ### normal samples have  an "N" prepended to the patient IDs. The
        ### reformat_normal_patient_ids function erases that and puts a ".N" at
        ### the end. See cptac/dataframe_tools.py for further function
        ### documentation.
        ###START EXAMPLE CODE###################################################
#        self._data = reformat_normal_patient_ids(self._data, existing_identifier="N", existing_identifier_location="start")
        ###END EXAMPLE CODE#####################################################

        # Call function from dataframe_tools.py to sort all tables first by sample status, and then by the index
#        self._data = sort_all_rows(self._data)

        # Call function from dataframe_tools.py to standardize the names of the index and column axes
#        self._data = standardize_axes_names(self._data)

        print(" " * len(formatting_msg), end='\r') # Erase the formatting message

==> cptac-0.9.1-py3-none-any.whl: cptac/pancan/umichov.py <==
#   Copyright 2018 Samuel Payne sam_payne@byu.edu
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#       http://www.apache.org/licenses/LICENSE-2.0
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import pandas as pd
import numpy as np
import os
import warnings
import datetime

from cptac.dataset import Dataset
from cptac.dataframe_tools import *
from cptac.exceptions import FailedReindexWarning, PublicationEmbargoWarning, ReindexMapError


class UmichOv(Dataset):

    def __init__(self, no_internet, version):
        """Load all of the umichov dataframes as values in the self._data dict variable, with names as keys, and format them properly.

        Parameters:
        version (str, optional): The version number to load, or the string "latest" to just load the latest building. Default is "latest".
        no_internet (bool, optional): Whether to skip the index update step because it requires an internet connection. This will be skipped automatically if there is no internet at all, but you may want to manually skip it if you have a spotty internet connection. Default is False.
        """

        # Set some needed variables, and pass them to the parent Dataset class __init__ function

        # This keeps a record of all versions that the code is equipped to handle. That way, if there's a new data release but they didn't update their package, it won't try to parse the new data version it isn't equipped to handle.
        valid_versions = ["1.0"]

        data_files = {
            "1.0": ["Report_abundance_groupby=protein_protNorm=MD_gu=2.tsv",
                    "OV_sample_TMT_annotation_UMich_GENCODE34_0315.csv",
                    "Report_abundance_groupby=multi-site_protNorm=MD_gu=2.tsv"
                #"S039_BCprospective_observed_0920.tsv.gz",
                #"S039_BCprospective_imputed_0920.tsv.gz"
            ]
        }

        # Call the parent class __init__ function
        super().__init__(cancer_type="umichov", version=version, valid_versions=valid_versions, data_files=data_files, no_internet=no_internet)

        # Load the data into dataframes in the self._data dict
        loading_msg = f"Loading {self.get_cancer_type()} v{self.version()}"
        for file_path in self._data_files_paths: # Loops through files variable

            # Print a loading message. We add a dot every time, so the user knows it's not frozen.
            loading_msg = loading_msg + "."
            print(loading_msg, end='\r')

            path_elements = file_path.split(os.sep) # Get a list of the levels of the path
            file_name = path_elements[-1] # The last element will be the name of the file. We'll use this to identify files for parsing in the if/elif statements below
            
            
            if file_name == "Report_abundance_groupby=protein_protNorm=MD_gu=2.tsv":
                df = pd.read_csv(file_path, sep = "\t") 
                df['Database_ID'] = df.Index.apply(lambda x: x.split('|')[0]) # Get protein identifier 
                df['Name'] = df.Index.apply(lambda x: x.split('|')[6]) # Get protein name 
                df = df.set_index(['Name', 'Database_ID']) # set multiindex
                df = df.drop(columns = ['Index', 'MaxPepProb', 'NumberPSM', 'Gene']) # drop unnecessary  columns
                df = df.transpose()
                ref_intensities = df.loc["ReferenceIntensity"] # Get reference intensities to use to calculate ratios 
                df = df.subtract(ref_intensities, axis="columns") # Subtract reference intensities from all the values
                df = df.iloc[1:,:] # drop ReferenceIntensity row 
                df.index.name = 'Patient_ID'
    
                # Drop quality control and ref intensity cols
                drop_cols = ['JHU-QC', 'JHU-QC.1', 'JHU-QC.2', 'JHU-QC.3', 'JHU-QC.4',
                           'RefInt_PNNL-JHU-Ref-1', 'RefInt_PNNL-JHU-Ref-2',
                           'RefInt_PNNL-JHU-Ref-3', 'RefInt_PNNL-JHU-Ref-4',
                           'RefInt_PNNL-JHU-Ref-5', 'RefInt_PNNL-JHU-Ref-6',
                           'RefInt_PNNL-JHU-Ref-7', 'RefInt_PNNL-JHU-Ref-8',
                           'RefInt_PNNL-JHU-Ref-9', 'RefInt_PNNL-JHU-Ref-10',
                           'RefInt_PNNL-JHU-Ref-11', 'RefInt_PNNL-JHU-Ref-12']
                df = df.drop(drop_cols, axis = 'index')
                self._data["proteomics"] = df
                
            elif file_name == "OV_sample_TMT_annotation_UMich_GENCODE34_0315.csv":
                ov_map = pd.read_csv(file_path, sep = ",", index_col = 0)
                ov_map = ov_map[['sample']].reset_index()
                ov_map = ov_map.replace("#",".", regex = True)
                self._helper_tables["map_ids"] = ov_map
                
            elif file_name == "Report_abundance_groupby=multi-site_protNorm=MD_gu=2.tsv":
                df = pd.read_csv(file_path, sep = "\t") 
                df[['Protein_ID','Transcript_ID',"Database_ID","Havana_gene","Havana_transcript","Transcript","Name","Site"]] = df.Index.str.split("\\|",expand=True)
                df[['num1','num2',"num3","num4","num5","Site"]] = df.Site.str.split("_",expand=True) 
                df = df[df['Site'].notna()] # only keep columns with phospho site 
                df = df.set_index(["Name","Database_ID","Peptide","Site"]) 
                #drop columns not needed in df 
                df.drop([ 'Gene', "Index","num1","num2","num3","num4","num5","Havana_gene","Havana_transcript","MaxPepProb","Protein_ID","Transcript_ID","Transcript"], axis=1, inplace=True)
                
                self._data["phosphoproteomics"] = df

            '''
            elif file_name == "S039_BCprospective_observed_0920.tsv.gz":
                df = pd.read_csv(file_path, sep="\t")
                df = df.transpose()
                df.index.name = 'Patient_ID'
                df.columns.name = 'Name'
                df = average_replicates(df)
                df = df.sort_values(by=["Patient_ID"])
                self._data["proteomics"] = df  
                
            elif file_name == "S039_BCprospective_imputed_0920.tsv.gz":
                df = pd.read_csv(file_path, sep="\t")
                df = df.transpose()
                df.index.name = 'Patient_ID'
                df.columns.name = 'Name'
                df = average_replicates(df)
                df = df.sort_values(by=["Patient_ID"])
                self._data["proteomics_imputed"] = df'''
            
        # This code may need to be deleted 
        # The '.1' added to the df is added by pd.read_csv() when there are duplicate columns
        # The '.1' after the read on Box is read in '#1' in the mapping file do not represent the same sample
        # Emailed about duplicate columns and waiting for response
        '''    
        # Proteomics
        # Get Patient_IDs
        # slice mapping_df to include cancer specific aliquot_IDs 
        prot = self._data["proteomics"]
        index_list = list(prot.index)
        mapping_df = self._data["map_ids"]
        cancer_df = mapping_df.loc[mapping_df['index'].isin(index_list)]
        # Create dictionary with aliquot_ID as keys and patient_ID as values
        matched_ids = {}
        for i, row in cancer_df.iterrows():
            matched_ids[row['index']] = row['sample']
        prot = prot.reset_index()
        prot = prot.replace(matched_ids) # replace aliquot_IDs with Patient_IDs
        prot = prot.set_index('Patient_ID')
          
        # average duplicates - CHECK really duplicates (some blanks in file are normal, 
        # but no blanks in file for these ids)
        replicate_IDs = ['01OV029', '15OV001', '17OV002']
        for rep in replicate_IDs:
            id_df = prot[prot.index.str.contains(rep)] 
            vals = list(id_df.mean(axis=0)) # average replicates and store in list 
            prot = prot.drop(index = rep) # drop both replicates so can add new row with averages
            prot.loc[rep] = vals'''
        
        prot = self._data["proteomics"]
        prot.index = prot.index.str.replace('-T$','', regex = True)
        prot.index = prot.index.str.replace('-N$','.N', regex = True)
        
        # Sort
        normal = prot.loc[prot.index.str.contains('\.N$', regex = True)]
        normal = normal.sort_values(by=["Patient_ID"])
        tumor = prot.loc[~ prot.index.str.contains('\.N$', regex = True)]
        tumor = tumor.sort_values(by=["Patient_ID"])
        all_prot = tumor.append(normal)  
        self._data["proteomics"] = all_prot
                
        
        ## phosphoproteomics 
        phos = self._data["phosphoproteomics"]
        ovmap = self._helper_tables["map_ids"]
        ovmap = ovmap.set_index("index")
        ovmap_dict = ovmap.to_dict()["sample"]
        
        phos = phos.rename(columns = ovmap_dict)# rename NAT ID columns with .N 
        phos = phos.T #transpose df 
        ref_intensities = phos.loc["ReferenceIntensity"]# Get reference intensities to use to calculate ratios 
        phos = phos.subtract(ref_intensities, axis="columns") # Subtract reference intensities from all the values, to get ratios
        phos = phos.iloc[1:,:] # drop ReferenceIntensity row 
        
        phos.index = phos.index.str.replace('-T$','', regex = True)
        phos.index = phos.index.str.replace('-N$','.N', regex = True)
        
        drop_cols_phos = ['JHU-QC','JHU-QC-1-Q','JHU-QC-2-Q','JHU-QC-3-Q','JHU-QC-4-Q','RefInt_PNNL-JHU-Ref-1-R',
 'RefInt_PNNL-JHU-Ref-2-R','RefInt_PNNL-JHU-Ref-3-R','RefInt_PNNL-JHU-Ref-4-R','RefInt_PNNL-JHU-Ref-5-R',
 'RefInt_PNNL-JHU-Ref-6-R','RefInt_PNNL-JHU-Ref-7-R','RefInt_PNNL-JHU-Ref-8-R','RefInt_PNNL-JHU-Ref-9-R',
 'RefInt_PNNL-JHU-Ref-10-R','RefInt_PNNL-JHU-Ref-11-R','RefInt_PNNL-JHU-Ref-12-R']

          # Drop quality control and ref intensity cols
        phos = phos.drop(drop_cols_phos, axis = 'index')
     
        # average duplicates - CHECK really duplicates (some blanks in file are normal, 
        # but no blanks in file for these ids)
        replicate_IDs = ['01OV029', '15OV001', '17OV002']
        for rep in replicate_IDs:
            id_df = phos[phos.index.str.contains(rep)] 
            vals = list(id_df.mean(axis=0)) # average replicates and store in list 
            phos = phos.drop(index = rep) # drop both replicates so can add new row with averages
            phos.loc[rep] = vals
            
        # Sort values
        phos.index.name = 'Patient_ID'
        normal = phos.loc[phos.index.str.contains('\.N$', regex = True)]
        normal = normal.sort_values(by=["Patient_ID"])
        tumor = phos.loc[~ phos.index.str.contains('\.N$', regex = True)]
        tumor = tumor.sort_values(by=["Patient_ID"])
        all_prot = tumor.append(normal)    
         
        self._data["phosphoproteomics"] = all_prot
        
        
        
        print(' ' * len(loading_msg), end='\r') # Erase the loading message
        formatting_msg = "Formatting dataframes..."
        print(formatting_msg, end='\r')

        # Get a union of all dataframes' indices, with duplicates removed
        ###FILL: If there are any tables whose index values you don't want
        ### included in the master index, pass them to the optional 'exclude'
        ### parameter of the unionize_indices function. This was useful, for
        ### example, when some datasets' followup data files included samples
        ### from cohorts that weren't in any data tables besides the followup
        ### table, so we excluded the followup table from the master index since
        ### there wasn't any point in creating empty representative rows for
        ### those samples just because they existed in the followup table.
#        master_index = unionize_indices(self._data) 

        # Use the master index to reindex the clinical dataframe, so the clinical dataframe has a record of every sample in the dataset. Rows that didn't exist before (such as the rows for normal samples) are filled with NaN.
#        new_clinical = self._data["clinical"]
#        new_clinical = new_clinical.reindex(master_index)

        # Add a column called Sample_Tumor_Normal to the clinical dataframe indicating whether each sample was a tumor or normal sample. Use a function from dataframe_tools to generate it.

        ###FILL: Your dataset should have some way that it marks the Patient IDs
        ### of normal samples. The example code below is for a dataset that
        ### marks them by putting an 'N' at the beginning of each one. You will
        ### need to write a lambda function that takes a given Patient_ID string
        ### and returns a bool indicating whether it corresponds to a normal
        ### sample. Pass that lambda function to the 'normal_test' parameter of
        ### the  generate_sample_status_col function when you call it. See 
        ### cptac/dataframe_tools.py for further function documentation.
        ###START EXAMPLE CODE###################################################
#        sample_status_col = generate_sample_status_col(new_clinical, normal_test=lambda sample: sample[0] == 'N')
        ###END EXAMPLE CODE#####################################################

#        new_clinical.insert(0, "Sample_Tumor_Normal", sample_status_col)

        # Replace the clinical dataframe in the data dictionary with our new and improved version!
#        self._data['clinical'] = new_clinical

        # Edit the format of the Patient_IDs to have normal samples marked the same way as in other datasets. 
        
        ###FILL: You will need to pass the proper parameters to correctly
        ### reformat the patient IDs in your dataset. The standard format is to
        ### have the string '.N' appended to the end of the normal patient IDs,
        ### e.g. the  normal patient ID corresponding to C3L-00378 would be
        ### C3L-00378.N (this way we can easily match two samples from the same
        ### patient). The example code below is for a dataset where all the
        ### normal samples have  an "N" prepended to the patient IDs. The
        ### reformat_normal_patient_ids function erases that and puts a ".N" at
        ### the end. See cptac/dataframe_tools.py for further function
        ### documentation.
        ###START EXAMPLE CODE###################################################
#        self._data = reformat_normal_patient_ids(self._data, existing_identifier="N", existing_identifier_location="start")
        ###END EXAMPLE CODE#####################################################

        # Call function from dataframe_tools.py to sort all tables first by sample status, and then by the index
#        self._data = sort_all_rows(self._data)

        # Call function from dataframe_tools.py to standardize the names of the index and column axes
#        self._data = standardize_axes_names(self._data)

        print(" " * len(formatting_msg), end='\r') # Erase the formatting message
